"""
Refers as pixCalc
"""
from typing import Tuple

import numpy as np
import cv2
from scipy.spatial import distance


class Pixel:

    @staticmethod
    def calc_origin(y0, x0, box, height=0, width=0, horizon=0, lefBegin=0, rightBegin=0):
        """
            basically it does the process of converting prediction from pictures taken as tile to
            pixel coordinates on panoramic images.
        Args:
            y0: tiled panoramic image column number
            x0: tiled panoramic image row number
            box: tiled image detected object bounding box
            height:
            width:
            horizon:
            lefBegin:
            rightBegin:

        Returns:

        """
        if len(box) == 4:
            xmin = x0 + int(round(box[0] * width + lefBegin))
            ymin = horizon + y0 + int(round(box[1] * height))
            xmax = x0 + int(round(box[2] * width + rightBegin))
            ymax = horizon + y0 + int(round(box[3] * height))
            return [xmin, ymin, xmax, ymax]
        elif len(box) == 2:
            coordx = x0 + int(round(box[0] * width + lefBegin))
            coordy = horizon + y0 + int(round(box[1] * height))
            return [coordx, coordy]

    @staticmethod
    def calc_image_tile(cropImage, numRows, numCols):
        tileHeight = int(cropImage.shape[0] / numRows)
        tileWidth = int(cropImage.shape[1] / numCols)
        return tileHeight, tileWidth

    @staticmethod
    def calc_average_bbox_coord(bbox_coords1, bbox_coords2):
        result = []
        [result.append((a + b) / 2) for a, b in zip(bbox_coords1, bbox_coords2)]
        return result

    @staticmethod
    def calc_bb_intersection_over_union(boxA, boxB):
        """
        :return: the intersection over union value
        """
        # determine the (x, y)-coordinates of the intersection rectangle
        xA = max(boxA[0], boxB[0])
        yA = max(boxA[1], boxB[1])
        xB = min(boxA[2], boxB[2])
        yB = min(boxA[3], boxB[3])
        # compute the area of intersection rectangle
        interArea = max(0, xB - xA + 1) * max(0, yB - yA + 1)
        # compute the area of both the prediction and ground-truth
        # rectangles
        boxAArea = (boxA[2] - boxA[0] + 1) * (boxA[3] - boxA[1] + 1)
        boxBArea = (boxB[2] - boxB[0] + 1) * (boxB[3] - boxB[1] + 1)
        # compute the intersection over union by taking the intersection
        # area and dividing it by the sum of prediction + ground-truth
        # areas - the interesection area
        iou = interArea / float(boxAArea + boxBArea - interArea)

        # TODO threshold will be dynamic
        if iou > 0.4:
            return True
        else:
            return False

    @staticmethod
    def get_angle_pca(detectedObject: np.ndarray) -> float:
        # extract the PCA from segmentation object

        img_gray = cv2.cvtColor(detectedObject, cv2.IMREAD_GRAYSCALE)  # convert to grayscale
        # _, thresh = cv2.threshold(img_gray, 255, 1, cv2.THRESH_BINARY_INV)
        imag_arr = np.array(img_gray)
        mat = np.argwhere(imag_arr == 255)
        mat[:, [0, 1]] = mat[:, [1, 0]]
        mat = np.array(mat).astype(np.float32)  # have to convert type for PCA
        m, e = cv2.PCACompute(mat, mean=np.array([]))

        center = tuple(m[0])
        center = (int(center[0]), int(center[1]))

        # endpoint1 = tuple(m[0] + e[0] * 100)
        # endpoint1 = (int(endpoint1[0]), int(endpoint1[1]))

        endpoint2 = tuple(m[0] + e[1] * 50)
        endpoint2 = (int(endpoint2[0]), int(endpoint2[1]))

        x = center
        y = (center[0], center[1] + 50)
        z = endpoint2

        b = distance.euclidean(x, y)
        c = distance.euclidean(x, z)
        a = distance.euclidean(y, z)
        cos_alpha = (b * b + c * c - a * a) / (2 * b * c)
        pca_angle = float(np.degrees(np.arccos(cos_alpha)))

        return pca_angle

    @staticmethod
    def calc_image_scale_limiter(detect_bbox1: list, detect_bbox2: list) -> bool:
        """
        This function check bbox scale ratio between two bbox
        """
        # print(f"First Bbox = {detect_bbox1}")
        # print(f"Second Bbox = {detect_bbox2}")

        first_w, first_h = detect_bbox1[2] - detect_bbox1[0], detect_bbox1[3] - detect_bbox1[1]
        second_w, second_h = detect_bbox2[2] - detect_bbox2[0], detect_bbox2[3] - detect_bbox2[1]

        if any(x < 0 for x in [first_w, first_h, second_w, second_h]):
            raise Exception("Check detected Bbox ")

        # half_scale_w, half_scale_h = first_w * 0.75, first_h * 0.75
        # double_scale_w, double_scale_h = first_w * 2.0, first_h * 2.0
        w_rate = first_w / second_w if first_w > second_w else second_w / first_w
        h_rate = first_h / second_h if first_h > second_h else second_h / first_h

        # eğer bbox width'i büyümüş ise height büyümesi beklenir ve bunlar maximum 2 katı oranında
        return True if 1 <= w_rate <= 2 and 1 <= h_rate <= 2 else False

    @staticmethod
    def check_bbox_size(box, cfg):
        """
        it's check bbox size from taken config params
        Args:
            box: detected objects bounding box
            cfg: criterion such as bboxHeight, bboxWidth

        Returns:

        """
        xmin, ymin, xmax, ymax = list(map(int, box))
        # print("Y Distances = ",np.abs(ymax-ymin),
        #       "X Distances = ", np.abs(xmax-xmin))
        if np.abs(ymax - ymin) < cfg.boundingBoxMinHeight \
                or np.abs(
            xmax - xmin) < cfg.boundingBoxMinWidth:  # these variables limited detected box in panoramic that only get this section.
            return False
        return True

    @staticmethod
    def calc_predict_tile(row: int, col: int, tileHeight: int, tileWidth: int, intersection_size: int,
                          height: int, width: int) -> Tuple:

        y0 = row * tileHeight
        y1 = y0 + tileHeight
        x0 = (col * tileWidth)
        x1 = x0 + tileWidth

        y1 = y1 + intersection_size
        x1 = x1 + intersection_size

        if row == 0 and col == 0:
            y0 = 0
            x0 = 0
            y1 = (y0 + tileHeight) + intersection_size
            x1 = (x0 + tileWidth) + intersection_size

        if y1 > height:
            y1 = height
            y0 = y0 - intersection_size

        if x1 > width:
            x1 = width
            x0 = x0 - intersection_size

        if x0 < 0:
            x0 = 0
        if y0 < 0:
            y0 = 0

        return x0, x1, y0, y1

    @staticmethod
    def middle_points_pixels(point_1: list, point_2: list) -> list:
        """
        :param point_1: List of contain a point's coordinates value
        :param point_2: List of contain a point's coordinates value
        :return: Middle point of between given two points
        """
        middle_p = [(point_1[0] + point_2[0]) / 2, (point_1[1] + point_2[1]) / 2]
        return middle_p

    @staticmethod
    def distance_pixels(cor_1: list, cor_2: list) -> int:
        from math import sqrt
        """
        :param cor_1: List of contain a point's coordinates value
        :param cor_2: List of contain a point's coordinates value
        :return: Distance between given two points
        """
        dist = int(sqrt((cor_2[0] - cor_1[0]) ** 2 + (cor_2[1] - cor_1[1]) ** 2))
        return dist


import numpy as np
import cv2
from helper.convertor import Convertor

class ImageTransformer(object):
    """ Perspective transformation class for image
        with shape (height, width, #channels) """

    def __init__(self, image):
        self.image = image

        self.height = self.image.shape[0]
        self.width = self.image.shape[1]
        self.num_channels = self.image.shape[2]

    """ Wrapper of Rotating a Image """

    def rotate_along_axis(self, pitch,roll=0, yaw=0, dx=0, dy=0, dz=1):
        # Get radius of rotation along 3 axes
        # roll : x-axis
        # pitch : y-axis
        # yaw : z-axis
        cc = Convertor()
        rpitch, rroll, ryaw = cc.get_rad(pitch, roll, yaw=0)

        # Get ideal focal length on z axis
        # NOTE: Change this section to other axis if needed
        d = np.sqrt(self.height ** 2 + self.width ** 2)
        self.focal = d / (2 * np.sin(yaw) if np.sin(yaw) != 0 else 1)
        dz = self.focal

        # Get projection matrix
        mat = self.get_M(rroll, rpitch, ryaw, dx, dy, dz)

        return cv2.warpPerspective(self.image.copy(), mat, (self.width, self.height))

    """ Get Perspective Projection Matrix """

    def get_M(self, roll, pitch, yaw, dx, dy, dz):
        w = self.width
        h = self.height
        f = self.focal

        # Projection 2D -> 3D matrix
        A1 = np.array([[1, 0, -w / 2],
                       [0, 1, -h / 2],
                       [0, 0, 1],
                       [0, 0, 1]])

        # Rotation matrices around the X, Y, and Z axis
        RX = np.array([[1, 0, 0, 0],
                       [0, np.cos(roll), -np.sin(roll), 0],
                       [0, np.sin(roll), np.cos(roll), 0],
                       [0, 0, 0, 1]])

        RY = np.array([[np.cos(pitch), 0, -np.sin(pitch), 0],
                       [0, 1, 0, 0],
                       [np.sin(pitch), 0, np.cos(pitch), 0],
                       [0, 0, 0, 1]])

        RZ = np.array([[np.cos(yaw), -np.sin(yaw), 0, 0],
                       [np.sin(yaw), np.cos(yaw), 0, 0],
                       [0, 0, 1, 0],
                       [0, 0, 0, 1]])

        # Composed rotation matrix with (RX, RY, RZ)
        R = np.dot(np.dot(RX, RY), RZ)
        # dz = (f - 0) / 1 ** 2

        # Translation matrix
        T = np.array([[1, 0, 0, dx],
                      [0, 1, 0, dy],
                      [0, 0, 1, dz],
                      [0, 0, 0, 1]])

        # Projection 3D -> 2D matrix
        A2 = np.array([[f, 0, w / 2, 0],
                       [0, f, h / 2, 0],
                       [0, 0, 1, 0]])

        # Final transformation matrix
        return np.dot(A2, np.dot(T, np.dot(R, A1)))



import urllib.request
import json
from typing import Dict, Any

from calculation_test.util import __name__ as name
from calculation_test.util import __major__ as major
from calculation_test.util import __minor__ as minor
from calculation_test.util import __patch__ as patch


def update_version(reqs: list, major: bool, minor: bool, patch:bool) -> str:
    version = {}
    version['major'] = reqs[0]
    version['minor'] = reqs[1]
    version['patch'] = reqs[2]
    print(f"Old Version = {version['major']}.{version['minor']}.{version['patch']}")
    if patch:
        version['patch'] += 1
    if minor:
        version['patch'] = 0
        version['minor'] += 1
    if major:
        version['patch'] = 0
        version['minor'] = 0
        version['major'] += 1
    new_version = str(version['major']) + "." + str(version['minor']) + "." + str(version['patch'])
    return new_version

def new_version(package: str, major: bool=False, minor:bool=False, patch:bool=True) -> str:
    req = urllib.request.Request(f'https://pypi.python.org/pypi/{package}/json')
    r = urllib.request.urlopen(req)
    if r.code == 200:
        t = json.loads(r.read())
        releases = t['info']['version']
        if releases:
            results = list(map(int, releases.split(".")))
            return update_version(results,major,minor,patch)


__version__ = new_version(name, major, minor, patch)



import math
from typing import Callable, Tuple, List

from calculation_test.distance import Distance
from helper.convertor import Convertor

from addict import Dict
import collections


class Intersection:

    def __init__(self, **kwargs):
        """

        Args:
            **kwargs:
            -> intersection_lineLength=self.config.intersection.lineLength,
            -> angle_wide=self.config.intersection.angle_wide
        """
        self.__dict__.update(kwargs)

        self.intersection_lineLength = self.intersection_rule.lineLength
        self.intersection_angle_wide = self.intersection_rule.angle_wide

    @staticmethod
    def apply_confidence_rule(c):
        """

        Args:
            c: detected car location

        Returns:
            it's score for how many has captured car locations and calculate 0 < result < 100
        """
        confidence_rate = (1 - 1 / c) if c else "{} can't null".format({c})

        return confidence_rate

    @staticmethod
    def angle_between(frameFilters: list) -> list:
        """
            calculated theta is it between define angle hands
        Args:
            # TODO car1 and car2 [ theta1, theta2, angleMIN, angleMAX]
            frameFilters:  car1 = [theta1, angleMIN, angleMAX], car2 = [theta1, angleMIN, angleMAX]
        Returns:
            only two position cars calculate and return such as [True, False],  [False, False]
        """
        rets = []
        for params in frameFilters:
            ## n = chech between a and b parameter
            ## a start point angle
            ## b finish point angle
            n = int(params[0])
            a = int(params[1])
            b = int(params[2])
            n = (360 + (n % 360)) % 360
            a = (3600000 + a) % 360
            b = (3600000 + b) % 360
            if a < b:
                result = a <= n <= b
                rets.append(result)
            else:
                result = a <= n or n <= b
                rets.append(result)

        return rets

    def calc_between_heading_angle(self, nArr: list) -> Dict:
        """
         angle with heading relations check for intersection angle wide
         Example1 :
            angle  = 120 and heading = 75  result {theta must be between max 195 min 45}
         Example2 :
            angle  = 50 and heading = 320  result {theta must be between max 270 min 30}
        Args:
            nArr: its list and include headings

        Returns:
            up limit and down limit for theta degrees.
        """

        a = self.intersection_angle_wide
        ret = []
        for n in nArr:
            t = 360 + n
            x = (-(a - t)) % 360
            y = (a + t) % 360
            # x, y hands of angle
            ret.append([x, y])

        anglesArr = [item for sublist in ret for item in sublist]
        assert len(anglesArr) >= 4, "Check your Thetas"
        angles = Dict({
            'max1Ahead': anglesArr[0],
            'min1Ahead': anglesArr[1],
            'max2Ahead': anglesArr[2],
            'min2Ahead': anglesArr[3]
        })
        del anglesArr
        return angles

    def ops_detect(self, loc):
        """
        location location of points
        intersection_lineLength: on the other hand, allows it to change automatically
        when we change it from the config file to specify the line length here.
        Args:
            loc: calculated coordinates

        Returns:

        """
        ops = []
        for i in range(0, len(loc), 3):
            ops.append(Distance.destination_point(lat=loc[i], lon=loc[i + 1], distance=self.intersection_lineLength,
                                                  bearing=loc[i + 2]))

        return ops[0], ops[1]

    @staticmethod
    def check_line_intersection(**kwargs) -> Dict:
        """
        Calculate from two points intersected locations

        Args:
            **kwargs:
            -> line1StartX
            -> line1StartY
            -> line1EndX
            -> line1EndY
            -> line2StartX
            -> line2StartY
            -> line2EndX
            -> line2EndY

        Returns:

        """

        points = Dict(kwargs)
        line1StartX = points.line1StartX
        line1StartY = points.line1StartY
        line1EndX = points.line1EndX
        line1EndY = points.line1EndY
        line2StartX = points.line2StartX
        line2StartY = points.line2StartY
        line2EndX = points.line2EndX
        line2EndY = points.line2EndY

        result = {
            "x": 0,
            "y": 0,
            "z": 0,
            "l1": False,
            "l2": False,
        }

        # We check whether any variable is decimal or not,
        # because the heading calculation_test in requests from the web can be decimal,
        # and the one from the database can be decimal.
        if isinstance(line1StartX, float):
            line1StartX, line1StartY, \
            line2StartX, line2StartY = intersection_float_to_decimal(line1StartX, line1StartY,
                                                                     line2StartX, line2StartY)
        if isinstance(line1EndX, float):
            line1EndX, line1EndY, \
            line2EndX, line2EndY = intersection_float_to_decimal(line1EndX, line1EndY,
                                                                 line2EndX, line2EndY)

        posit1 = ((line2EndY - line2StartY) * (line1EndX - line1StartX))
        posit2 = ((line2EndX - line2StartX) * (line1EndY - line1StartY))
        denominator = posit1 - posit2

        if denominator == 0:
            return result

        a = line1StartY - line2StartY
        b = line1StartX - line2StartX
        numerator1 = ((line2EndX - line2StartX) * a) - ((line2EndY - line2StartY) * b)
        numerator2 = ((line1EndX - line1StartX) * a) - ((line1EndY - line1StartY) * b)
        a = numerator1 / denominator
        b = numerator2 / denominator

        # if we cast these lines infinitely in both directions, they intersect here:
        result['x'] = line1StartX + (a * (line1EndX - line1StartX))
        result['y'] = line1StartY + (a * (line1EndY - line1StartY))

        if 0 < a < 1:
            result["l1"] = True

        if 0 < b < 1:
            result["l2"] = True

        return result

    def intersection_points_average(self, groupMatches: list, geojsonFormatFunc: Callable) -> Tuple[
        Dict, list, list, List[list]]:
        """

        Args:
            groupMatches: matched objects same class, same points
            geojsonFormatFunc: it's function for creating geojson format

        Returns:
            It calculates a single point by taking the averages of all the detected intersections.
        """
        total = Dict({
            'isValid': False
        })
        pointsMerged, matched_object_id, matched_paired = [], [], []

        objects = collections.defaultdict(list)
        i = 0
        confidence = 0
        for matches in groupMatches:
            i += 2
            matches = Dict(matches)
            objects['Lat_center'].append((matches.intersectCenter['x']))
            objects['Lon_center'].append((matches.intersectCenter['y']))
            objects['Alt_center'].append((matches.intersectCenter['z']))

            objects['Lat_cornerA'].append((matches.intersectCornerA['x']))
            objects['Lon_cornerA'].append((matches.intersectCornerA['y']))
            objects['Alt_cornerA'].append((matches.intersectCornerA['z']))

            objects['Lat_cornerB'].append((matches.intersectCornerB['x']))
            objects['Lon_cornerB'].append((matches.intersectCornerB['y']))
            objects['Alt_cornerB'].append((matches.intersectCornerB['z']))

            objects['Lat_cornerC'].append((matches.intersectCornerC['x']))
            objects['Lon_cornerC'].append((matches.intersectCornerC['y']))
            objects['Alt_cornerC'].append((matches.intersectCornerC['z']))

            objects['Lat_cornerD'].append((matches.intersectCornerD['x']))
            objects['Lon_cornerD'].append((matches.intersectCornerD['y']))
            objects['Alt_cornerD'].append((matches.intersectCornerD['z']))

            objects['avg_score'].append(float(matches.score_1))
            objects['avg_score'].append(float(matches.score_2))

            total['detectedPath_1'] = matches.detectedPath_1
            total['detectedPath_2'] = matches.detectedPath_2

            total['imgUrl_1'] = matches.imgUrl_1
            total['imgUrl_2'] = matches.imgUrl_2

            total['objId_1'] = matches.objId_1
            total['objId_2'] = matches.objId_2
            total['match_id'] = matches.match

            total['classname'] = matches.classname_1  # doesn't matter same classname_1 and classname_2

            total['feature'] = matches.feature_1 or matches.feature_2

            # creating geo json format according to paired points
            geojsonParams = geojsonFormatFunc(matches, type="Point")

            pointsMerged.append(geojsonParams)
            confidence = self.apply_confidence_rule(i)

            total['isValid'] = i > 0

            # added because of visualization matched paired objects
            matched_object_id.append(matches.objId_1)
            matched_object_id.append(matches.objId_2)
            matched_paired.append([matches.intersectCenter['x'],
                                   matches.intersectCenter['y']])

        for k, v in objects.items():
            total[k] = float(sum(v) / len(v))

        total['confidence'] = (total['avg_score'] + confidence) * 0.5
        del objects

        return total, pointsMerged, matched_object_id, matched_paired

    def intersection_points_find(self, **kwargs):
        """
        from two points calculate intersection
        Args:
            **kwargs:
            -> start_lat1
            -> start_lon1
            -> theta1
            -> start_lat2
            -> start_lon2
            -> theta2
            -> type : intersect or area

        Returns:
            get coordinates
        """
        points = Dict(kwargs)

        if points.type == "intersect":
            ops1, ops2 = self.ops_detect(loc=[points.start_lat1, points.start_lon1, points.theta1,
                                              points.start_lat2, points.start_lon2, points.theta2])

            destinationPoint1 = [(points.start_lat1, points.start_lon1),
                                 (ops1["lat"], ops1["lon"])]

            destinationPoint2 = [(points.start_lat2, points.start_lon2),
                                 (ops2["lat"], ops2["lon"])]

            interSection = self.check_line_intersection(line1StartX=points.start_lat1,
                                                        line1StartY=points.start_lon1,
                                                        line1EndX=ops1["lat"],
                                                        line1EndY=ops1["lon"],

                                                        line2StartX=points.start_lat2,
                                                        line2StartY=points.start_lon2,
                                                        line2EndX=ops2["lat"],
                                                        line2EndY=ops2["lon"])

            return interSection, destinationPoint1, destinationPoint2

        if points.type == "area":
            corners = {}
            for th1, th2, ph1, ph2, corner_id in zip(points.theta1, points.theta2,
                                                     points.phi1, points.phi2,
                                                     points.corners_id):
                ops1, ops2 = self.ops_detect(loc=[points.start_lat1, points.start_lon1, th1,
                                                  points.start_lat2, points.start_lon2, th2])

                is_intersect = self.check_line_intersection(line1StartX=points.start_lat1,
                                                            line1StartY=points.start_lon1,
                                                            line1EndX=ops1["lat"],
                                                            line1EndY=ops1["lon"],

                                                            line2StartX=points.start_lat2,
                                                            line2StartY=points.start_lon2,
                                                            line2EndX=ops2["lat"],
                                                            line2EndY=ops2["lon"]
                                                            )
                if is_intersect['l1'] and is_intersect['l2']:

                    distance_between_panoroma_first_and_intersected_point = Distance.haversine(
                        lon1=points.start_lon1, lat1=points.start_lat1,
                        lon2=is_intersect['y'], lat2=is_intersect['x'])

                    center_altA = (math.tan(
                        math.radians(ph1) * distance_between_panoroma_first_and_intersected_point) +
                                   float(points.start_alt1))

                    distance_between_panoroma_second_and_intersected_point = Distance.haversine(
                        lon1=points.start_lon2, lat1=points.start_lat2,
                        lon2=is_intersect['y'], lat2=is_intersect['x'])

                    center_altB = (math.tan(
                        math.radians(ph2) * distance_between_panoroma_second_and_intersected_point) +
                                   float(points.start_alt2))

                    is_intersect['z'] = (center_altA + center_altB) / 2

                corners[corner_id] = is_intersect

            return corners



def intersection_float_to_decimal(lat1, lon1, lat2, lon2) -> Tuple:
    """

    Args:
        lat1:
        lon1:
        lat2:
        lon2:

    Returns:
        points convert decimal format
    """

    lat1 = Convertor.decimal_fix(lat1)
    lon1 = Convertor.decimal_fix(lon1)
    lat2 = Convertor.decimal_fix(lat2)
    lon2 = Convertor.decimal_fix(lon2)

    return lat1, lon1, lat2, lon2


import math


class Area:

    # Deprecated
    @staticmethod
    def area_measurement(interSection_A_lat, interSection_A_lon,
                         interSection_B_lat, interSection_B_lon,
                         interSection_C_lat, interSection_C_lon):
        pass

    @staticmethod
    def calc_area_heron(a: float, b: float, c: float) -> float:
        """
             A _______ B
            |    X    |
            D _______ C

        Args:
            a: triangle edge ABC
            b: triangle edge BCD
            c: triangle edge CAD

        Returns:
            area float
        """

        s = (a + b + c) / 2
        return math.sqrt(abs(s * (s - a) * (s - b) * (s - c)))


class Iterator:

    @staticmethod
    def previous_current_next(iterable):
        """
        Make an iterator that yields an (previous, current, next) tuple per element.
        Args:
            iterable:

        Returns:
            None if the value does not make sense (i.e. previous before first and next after last).
        """
        iterable = iter(iterable)
        prv = None
        cur = iterable.__next__()
        try:
            while True:
                nxt = iterable.__next__()
                if abs(cur - nxt) > 1:
                    data = None
                else:
                    data = nxt
                if prv is not None:
                    if abs(prv - cur) > 1:
                        prv = None

                yield prv, cur, data
                prv = cur
                cur = nxt
        except StopIteration:
            if prv is not None:
                if abs(prv - cur) > 1:
                    prv = None
            yield prv, cur, None




from math import radians, cos, sin, asin, sqrt, atan2, pi
from calculation_test.units import TO_RAD, TO_DEG, RADIUS
from trianglesolver import solve, degree
import numpy
import math

from addict import Dict


class Distance:

    def __init__(self):
        pass

    @staticmethod
    def checkValidity(a: float, b: float, c: float) -> bool:
        """
        Triangle rule checks it's triangle edged as a, b, c
        Args:
            a: edge
            b: edge
            c: edge
        Returns:
            if is not triangle and C degree over the defined variable return False.
        """
        a, b, c, A, B, C = solve(a, b, c)
        if (a + b <= c) or (a + c <= b) or (b + c <= a):
            if not 5 < (C / degree) < 20:
                return False
        return True

    @staticmethod
    def bearing(**kwargs):
        """
        Calculate bearing(direction) from two points
        Args:
            **kwargs:
             -> startLat
             -> startLon
             -> destLat
             -> destLon
        Returns:
            Angle calculate from two point
        """
        """
        

        :param startLat:
        :param startLon:
        :param destLat:
        :param destLon:
        :return:
        """

        points = Dict(kwargs)
        startLat = points.startLat
        startLon = points.startLon
        destLat = points.destLat
        destLon = points.destLon

        phi1 = radians(startLat)
        phi2 = radians(destLat)
        cosPhi2 = cos(phi2)
        dLmd = radians(destLon - startLon)
        aci = atan2(sin(dLmd) * cosPhi2,
                    cos(phi1) * sin(phi2) - sin(phi1) * cosPhi2 * cos(dLmd))
        print(aci * TO_DEG)
        return aci * TO_DEG

    @staticmethod
    def haversine(**kwargs):
        """
        Calculate the great circle distance between two points
        on the earth (specified in decimal degrees)
        Args:
            **kwargs:
            -> lon1
            -> lat1
            -> lon2
            -> lat2

        Returns:
            distance between two points
        """
        points = Dict(kwargs)
        lon1 = points.lon1
        lat1 = points.lat1
        lon2 = points.lon2
        lat2 = points.lat2

        # convert decimal degrees to radians
        lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])

        # haversine formula
        dlon = lon2 - lon1
        dlat = lat2 - lat1
        a = sin(dlat / 2) ** 2 + cos(lat1) * cos(lat2) * sin(dlon / 2) ** 2
        c = 2 * asin(sqrt(a))
        # r = 6371  # Radius of earth in kilometers. Use 3956 for miles
        return c * RADIUS

    @staticmethod
    def destination_point(lat, lon, distance, bearing):
        """
        :lat :
        :lon :
        :distance :
        bearing :
        """

        δ = distance / RADIUS
        θ = bearing * TO_RAD
        φ1 = radians(lat)
        λ1 = radians(lon)
        φ2 = asin(sin(φ1) * cos(δ) + cos(φ1) * sin(δ) * cos(θ))
        λ2 = λ1 + atan2(sin(θ) * sin(δ) * cos(φ1), cos(δ) - sin(φ1) * sin(φ2))
        λ2 = (λ2 + 3 * pi) % (2 * pi) - pi

        return {"lat": φ2 * TO_DEG, "lon": λ2 * TO_DEG}

    def check_triangle_rule(self, cornerA, cornerB, cornerC) -> bool:
        """
            abs(edgeA - edgeB) < edgeC < edgeA + edgeB
        Args:
            cornerA: edge
            cornerB: edge
            cornerC: edge

        Returns:
            true or false
        """
        edgeAB = (self.haversine(lon1=cornerA[0], lat1=cornerA[1],
                                 lon2=cornerB[0], lat2=cornerB[1])) * 1000
        edgeAC = (self.haversine(lon1=cornerA[0], lat1=cornerA[1],
                                 lon2=cornerC[0], lat2=cornerC[1])) * 1000
        edgeBC = (self.haversine(lon1=cornerB[0], lat1=cornerB[1],
                                 lon2=cornerC[0], lat2=cornerC[1])) * 1000

        return self.checkValidity(edgeAB, edgeAC, edgeBC)



    @staticmethod
    def line_to_XYs(line):
        # return first and last coordinates
        firstX, firstY = (line.firstPoint.X, line.firstPoint.Y)
        lastX, lastY = (line.lastPoint.X, line.lastPoint.Y)
        return [(firstX, firstY), (lastX, lastY)]

    @staticmethod
    def calculate_initial_compass_bearing(pointA, pointB):
        """
        Calculates the bearing between two points.
        The formulae used is the following:
            θ = atan2(sin(Δlong).cos(lat2),
                      cos(lat1).sin(lat2) − sin(lat1).cos(lat2).cos(Δlong))
        Args:
            pointA: The tuple representing the latitude/longitude for the
                    first point. Latitude and longitude must be in decimal degrees
            pointB: The tuple representing the latitude/longitude for the
                    second point. Latitude and longitude must be in decimal degrees

        Returns:
            Angle calculate from two point
        """
        if (type(pointA) != tuple) or (type(pointB) != tuple):
            raise TypeError("Only tuples are supported as arguments")

        lat1 = math.radians(pointA[0])
        lat2 = math.radians(pointB[0])

        diffLong = math.radians(pointB[1] - pointA[1])

        x = math.sin(diffLong) * math.cos(lat2)
        y = math.cos(lat1) * math.sin(lat2) - (math.sin(lat1)
                                               * math.cos(lat2) * math.cos(diffLong))

        initial_bearing = math.atan2(x, y)

        # Now we have the initial bearing but math.atan2 return values
        # from -180° to + 180° which is not what we want for a compass bearing
        # The solution is to normalize the initial bearing as shown below
        initial_bearing = math.degrees(initial_bearing)
        compass_bearing = (initial_bearing + 360) % 360

        return compass_bearing

    @staticmethod
    def distance_between_points(origin, destination):

        lat1, lon1 = origin
        lat2, lon2 = destination
        radius = 6371  # km

        dlat = math.radians(lat2 - lat1)
        dlon = math.radians(lon2 - lon1)
        a = math.sin(dlat / 2) * math.sin(dlat / 2) + math.cos(math.radians(lat1)) \
            * math.cos(math.radians(lat2)) * math.sin(dlon / 2) * math.sin(dlon / 2)
        c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
        d = radius * c

        return d



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['calculation_test']

package_data = \
{'': ['*']}

install_requires = \
['ExifRead>=2.3.2,<3.0.0',
 'addict==2.4.0',
 'geographiclib==1.50',
 'helper-mapilio>=1.1.31,<2.0.0',
 'matplotlib>=3.5.2,<4.0.0',
 'numpy>=1.23.1,<2.0.0',
 'opencv-python>=4.5.1.48,<5.0.0.0',
 'pandas',
 'pytz>=2021.1,<2022.0',
 'requests>=2.25.1,<3.0.0',
 'scipy>=1.8.1,<2.0.0',
 'solve>=0.0.0,<0.0.1',
 'trianglesolver>=1.2,<2.0']

setup_kwargs = {
    'name': 'calculation-test',
    'version': '0.1.1',
    'description': 'mapilio calculation',
    'long_description': '# DENEME\n',
    'author': 'Kadir Coşar',
    'author_email': 'kadircosarnew@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import requests
from calculation_test.distance import Distance
from io import BytesIO
from helper.exif_read import ExifRead
from helper.generator import Generator


class ExifExtractHeading:
    # TODO pointA and pointB operations will be move `helper-mapilio`
    # TODO and these variables will send heading calc function.
    @staticmethod
    def heading_calc(img_urls: list) -> dict:
        processing_format = {}
        objects_zone = []
        for key, x in enumerate(range(0, len(img_urls) - 1, 1)):
            head_url = img_urls[x]['img_url']
            tail_url = img_urls[x + 1]['img_url']
            r_head = requests.get(
                head_url)
            r_tail = requests.get(
                tail_url)
            img1_exifData = ExifRead(BytesIO(r_head.content), details=True)
            img2_exifData = ExifRead(BytesIO(r_tail.content), details=True)
            img_info_1 = Generator.get_exif_information(img1_exifData)
            img_info_2 = Generator.get_exif_information(img2_exifData)
            img_info_1['key'] = key
            img_info_2['key'] = key + 1
            pointA = (img_info_1["coordx"], img_info_1["coordy"])
            pointB = (img_info_2["coordx"], img_info_2["coordy"])
            try:
                heading = Distance.calculate_initial_compass_bearing(pointA,
                                                                     pointB)  # calculate heading from two images
            except ValueError as v:
                raise Exception(f" {v} : Check the image Exif Data some missing values")

            img_info_1['heading'] = heading
            img_info_2['heading'] = heading
            img_info_1['img_url'] = head_url
            img_info_2['img_url'] = tail_url
            objects_zone.append(img_info_1)
            if len(img_urls) - 1 == x + 1:  # if last indices set: previous heading
                objects_zone.append(img_info_2)

        processing_format["zone"] = objects_zone

        return processing_format


import logging
from decimal import Decimal

NUMBER_TYPES = (int, float, Decimal)

__version__ = "0.1.12"
__name__ = "calculation_test-mapilio"
__major__ = False
__minor__ = False
__patch__ = True


logger = logging.getLogger('calculation_test')

